"use strict";

exports.__esModule = true;
var nonWhitespace = /\S/;
var newlines = /\r\n?|\n/;

// Trims the whitespace off the lines.
function lineFilter(lines, line, i, _ref2) {
  var length = _ref2.length;

  if (i > 0) {
    line = line.trimLeft();
  }
  if (i + 1 < length) {
    line = line.trimRight();
  }
  if (line) {
    lines.push(line);
  }

  return lines;
}

// Cleans the whitespace from a text node.
function cleanText(_ref3) {
  var value = _ref3.value;

  if (nonWhitespace.test(value)) {
    var lines = value.split(newlines);

    lines = lines.reduce(lineFilter, []);

    return lines.join(" ");
  }
}

exports["default"] = function (t) {

  // Helper to create a function call in AST.
  function toFunctionCall(functionName, args) {
    return t.callExpression(t.identifier(functionName), args);
  }

  // Helper to create a function call statement in AST.
  function toFunctionCallStatement(functionName, args) {
    return t.expressionStatement(toFunctionCall(functionName, args));
  }

  // Helper to transform a JSX identifier into a normal reference.
  function toReference(node, identifier) {
    if (t.isJSXIdentifier(node)) {
      return identifier ? t.identifier(node.name) : t.literal(node.name);
    }
    return t.memberExpression(toReference(node.object, true), toReference(node.property, true));
  }

  // Filters out empty children, and transform JSX expressions
  // into normal expressions.
  function childFilter(children, child) {
    if (t.isJSXExpressionContainer(child)) {
      child = child.expression;
    }

    if (t.isLiteral(child) && typeof child.value === "string") {
      var text = cleanText(child);
      if (!text) {
        return children;
      }

      child = toFunctionCallStatement("text", [t.literal(text)]);
    }

    if (t.isJSXExpressionContainer(child)) child = child.expression;
    if (t.isJSXEmptyExpression(child)) {
      return children;
    } else if (t.isIdentifier(child)) {
      child = toFunctionCallStatement("text", [child]);
    } else if (t.isArrayExpression(child)) {
      child = t.sequenceExpression(buildChildren(child.elements));
    }

    children.push(child);
    return children;
  }

  // Helper to transform a call expression into an expression statement.
  function callToStatement(expression) {
    if (t.isCallExpression(expression)) {
      return t.expressionStatement(expression);
    }
    return expression;
  }

  // Helper to flatten out sequence expressions into a top level
  // expression statements.
  function sequenceReducer(nodes, node) {
    if (t.isSequenceExpression(node)) {
      var expressions = flattenExpressions(node.expressions);
      nodes.push.apply(nodes, expressions);
    } else {
      nodes.push(callToStatement(node));
    }
    return nodes;
  }

  function buildChildren(children) {
    return children.reduce(childFilter, []);
  }

  function flattenExpressions(expressions) {
    return expressions.reduce(sequenceReducer, []);
  }

  return {
    buildChildren: buildChildren,

    flattenExpressions: flattenExpressions,

    toFunctionCall: toFunctionCall,

    toFunctionCallStatement: toFunctionCallStatement,

    toReference: toReference,

    // Extracts attributes into the appropriate
    // attribute array. Static attributes and the key
    // are placed into static attributes, and expressions
    // are placed into the variadic attributes.
    extractOpenArguments: function extractOpenArguments(attributes) {
      var key = null;
      var statics = [];
      var attrs = [];
      var hasSpread = false;

      for (var _iterator = attributes, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var attribute = _ref;

        if (t.isJSXSpreadAttribute(attribute)) {
          hasSpread = true;
          attrs.push(attribute);
        } else {
          var _name = attribute.name.name;
          var attr = t.literal(_name);
          var value = attribute.value;

          if (!value) {
            value = t.literal(true);
          } else if (t.isJSXExpressionContainer(value)) {
            value = value.expression;
          }

          if (_name === "key") {
            key = value;
          }

          if (_name === "key" || t.isLiteral(value)) {
            statics.push(attr, value);
          } else {
            attrs.push([attr, value]);
          }
        }
      }

      if (!statics.length) {
        statics = null;
      }
      if (!attrs.length) {
        attrs = null;
      }

      return { key: key, statics: statics, attrs: attrs, hasSpread: hasSpread };
    },

    // Transforms an attribute array into sequential attr calls.
    attrsToAttrCalls: function attrsToAttrCalls(scope) {
      return function (attr) {
        if (t.isJSXSpreadAttribute(attr)) {
          var iterator = scope.generateUidIdentifier("attr");

          return t.forInStatement(t.variableDeclaration("var", [iterator]), attr.argument, toFunctionCallStatement("attr", [iterator, t.memberExpression(attr.argument, iterator, true)]));
        } else {
          return toFunctionCall("attr", attr);
        }
      };
    }
  };
};

;
module.exports = exports["default"];